# Plugin to add flatpakBuild XMLRPC command
#
# The code here is partially based on the code of Koji's kojihub.py
# which has the following license notice:
#
#    Koji is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation;
#    version 2.1 of the License.
#
#    This software is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this software; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#       Mike McLean <mikem@redhat.com>
#       Mike Bonnet <mikeb@redhat.com>
#       Cristian Balint <cbalint@redhat.com>
#
# Plugin author:
#       Owen Taylor <otaylor@redhat.com

import json
from typing import TYPE_CHECKING


from koji.context import context
from kojihub import (  # type: ignore
    convert_value,
    Host,
    list_archives,
    make_task,
    Task,
)
if TYPE_CHECKING:
    from koji.db import UpdateProcessor
else:
    from kojihub.db import UpdateProcessor
from koji.plugin import export
import koji.policy


@export
def flatpakBuild(src, target, opts=None, priority=None, channel=None):
    """Create a build task for a Flatpak

    priority: the amount to increase (or decrease) the task priority, relative
                to the default priority; higher values mean lower priority; only
                admins have the right to specify a negative priority here
    channel: the channel to allocate the task to
    Returns the task id
    """
    context.session.assertLogin()
    if not opts:
        opts = {}
    taskOpts = {}
    convert_value(src, cast=str, check_only=True)
    if priority:
        if priority < 0:
            if not context.session.hasPerm('admin'):
                raise koji.ActionNotAllowed('only admins may create high-priority tasks')
        taskOpts['priority'] = koji.PRIO_DEFAULT + priority
    if channel:
        taskOpts['channel'] = channel
    return make_task('flatpakBuild', [src, target, opts], **taskOpts)


@export
def flatpakSetExtra(task_id, build_id, build_extra, archive_extra):
    """set image data for Flatpak result
    """
    convert_value(build_id, cast=int, check_only=True)
    convert_value(build_extra, cast=dict, check_only=True)
    convert_value(archive_extra, cast=dict, check_only=True)

    host = Host()
    host.verify()
    task = Task(task_id)
    task.assertHost(host.id)

    # get typeinfo
    build_extra.setdefault('typeinfo', {})
    typeinfo = build_extra['typeinfo']

    # legacy types can be at top level of extra
    for btype in ['maven', 'win', 'image']:
        if btype not in build_extra:
            continue
        if btype in typeinfo:
            # he says they've already got one
            raise koji.GenericError(f"Duplicate typeinfo for {btype}")
        typeinfo[btype] = build_extra[btype]

    update = UpdateProcessor('build', clauses=['id=%(id)s'], values={
        'id': build_id,
    })
    update.set(extra=json.dumps(build_extra))
    update.execute()

    archives = list_archives(buildID=build_id)
    archive_map = {
        a['filename']: a['id'] for a in archives
    }

    for filename, extra in archive_extra.items():
        archive_id = archive_map.get(filename)
        if archive_id is None:
            raise koji.GenericError("No archive with filename='{filename}' for build {build_id}")

        update = UpdateProcessor('archiveinfo', clauses=['id=%(id)s'], values={
            'id': archive_id
        })
        update.set(extra=json.dumps(extra))
        update.execute()
